<?php

/**
 * @file
 * User page callbacks for the custom_search module.
 */

/*
 * Presents links to filter the search results.
 */
function custom_search_preprocess_item_list__search_results(&$variables) {
  $config = Drupal::config('custom_search.results')->getRawData();
  $page_config = FALSE;
  $path = substr(\Drupal::request()->getPathInfo(), 8);

  // Search for the config of this page
  foreach ($config as $c) {
    if ($c['path'] == $path) {
      $page_config = $c;
      break;
    }
  }
  
  $types = array();
  foreach ($variables['items'] as $key => $item) {
    // Render results.
    $variables['items'][$key]['value'] = drupal_render($item);
    // Count results for the filter.
    if ($page_config && $page_config['filter']['position'] != 'disabled') {
      $bundle = $item['#result']['node']->bundle();
      if (!array_key_exists($bundle, $types)) {
        $types[$bundle] = 1;
      }
      else {
        $types[$bundle]++;
      }
    }
  }

  if ($page_config) {

    $variables['filter_position'] = $page_config['filter']['position'];

    // Save # of results for collapsing advanced search.
    \Drupal::state()->set('custom_search.nb_results', count($variables['items']));

    // Generate the filter.
    if ($variables['filter_position'] != 'disabled') {
      // Get search arguments.
      $query = \Drupal::request()->server->get('QUERY_STRING');
      $query = explode('&', $query);
      $keys = array();
      $filters = array();
      foreach ($query as $key) {
        $key_exploded = explode('=', $key);
        // Exclude current types filters.
        if (isset($key_exploded[1])) {
          if (substr(urldecode($key_exploded[0]), 0, 2) == 'f[') {
            if (substr($key_exploded[1], 0, 4) != 'type') {
              $filters[] = $key_exploded[1];
            }
          }
          else {
            $keys[$key_exploded[0]] = $key_exploded[1];
          }
        }
      }
      for ($f = 0; $f < count($filters); $f++) {
        $keys['f[' . $f . ']'] = $filters[$f];
      }
      // Build menu.
      $node_types = node_type_get_names();
      $filter_items = array();
      $filter_items[] = l($page_config['filter']['any'], 'search/' . $page_config['path'], array('query' => $keys));
      foreach ($types as $type => $num_results) {
        $filter_items[] = l($node_types[$type], 'search/' . $page_config['path'], array('query' => array_merge($keys, array('f[' . count($filters) . ']' => 'type:' . $type))));
      }
      if (count($filter_items) > 1) {
        $filter_list = array(
          '#theme'      => 'item_list',
          '#title'      => $page_config['filter']['label'],
          '#items'      => $filter_items,
          '#list_type'  => 'ul',
          '#attributes' => array(
            'class' => array(
              'search-filter'
            ),
          ),
        );
        $variables['filter'] = drupal_render($filter_list);
      }
    }

  }
}

/**
 * Implements hook_theme_suggestions_HOOK().
 */
function custom_search_theme_suggestions_item_list__search_results(array $variables) {
  return array('item_list__custom_search_results__' . $variables['plugin_id']);
}

/*
 * Customisation of the results info.
 */
function custom_search_preprocess_search_result(&$variables) {
  $config = Drupal::config('custom_search.results')->getRawData();
  $page_config = FALSE;
  $path = substr(\Drupal::request()->getPathInfo(), 8);

  // Search for the config of this page
  foreach ($config as $c) {
    if ($c['path'] == $path) {
      $page_config = $c;
      break;
    }
  }

  if ($page_config) {
    $infos = array();
    if (in_array('type', $page_config['info'], TRUE) && $variables['plugin_id'] == 'node_search') {
      $infos[] = $variables['result']['type'];
    }
    if (isset($variables['info_split'])) {
      foreach ($variables['info_split'] as $key => $info) {
        if (in_array($key, $page_config['info'], TRUE)) {
          array_push($infos, $info);
        }
      }
    }
    $variables['info'] = implode(' - ', $infos);
  }

}

/**
 * Implements hook_theme_suggestions_HOOK().
 */
function custom_search_theme_suggestions_search_result(array $variables) {
  return array('custom_search_result__' . $variables['plugin_id']);
}
